from __future__ import absolute_import, unicode_literals


class AntiUserAgentError(Exception):
    pass


class HeaderDeprecationWarning(Warning):
    pass


class NotFoundParamError(Exception):
    pass


class UnSupportMethodError(Exception):
    pass


UserAgentError = AntiUserAgentError





def headers_dict_to_raw(headers_dict):
    r"""
    Returns a raw HTTP headers representation of headers

    For example:

    >>> import w3lib.http
    >>> w3lib.http.headers_dict_to_raw({b'Content-type': b'text/html', b'Accept': b'gzip'}) # doctest: +SKIP
    'Content-type: text/html\\r\\nAccept: gzip'
    >>>

    Note that keys and values must be bytes.

    Argument is ``None`` (returns ``None``):

    >>> w3lib.http.headers_dict_to_raw(None)
    >>>

    """

    if headers_dict is None:
        return None
    raw_lines = []
    for key, value in headers_dict.items():
        if isinstance(value, bytes):
            raw_lines.append(b": ".join([key, value]))
        elif isinstance(value, (list, tuple)):
            for v in value:
                raw_lines.append(b": ".join([key, v]))
    return b'\r\n'.join(raw_lines)



from __future__ import absolute_import, unicode_literals

import sys
import socket

from anti_useragent.utils import misc


try:
    from loguru import logger
except:
    misc.install("loguru")
    from loguru import logger


def set_log_config(formatter, logfile=None):
    return {
        "default": {
            "handlers": [
                {
                    "sink": sys.stdout,
                    "format": formatter,
                    "level": "TRACE"
                },
                {
                    "sink": "info.log" if not logfile else logfile,
                    "format": formatter,
                    "level": "INFO",
                    "rotation": '1 week',
                    "retention": '30 days',
                    'encoding': 'utf-8'
                },
            ],
            "extra": {
                "host": socket.gethostbyname(socket.gethostname()),
                'log_name': 'default',
                'type': 'None'
            },
            "levels": [
                dict(name="TRACE", icon="✏️", color="<cyan><bold>"),
                dict(name="DEBUG", icon="❄️", color="<blue><bold>"),
                dict(name="INFO", icon="♻️", color="<bold>"),
                dict(name="SUCCESS", icon="✔️", color="<green><bold>"),
                dict(name="WARNING", icon="⚠️", color="<yellow><bold>"),
                dict(name="ERROR", icon="❌️", color="<red><bold>"),
                dict(name="CRITICAL", icon="☠️", color="<RED><bold>"),
            ]
        },
        'kafka': True
    }


class LogFormatter(object):
    default_formatter = '<green>{time:YYYY-MM-DD HH:mm:ss,SSS}</green> | ' \
                        '[<cyan>{extra[log_name]}</cyan>] <cyan>{module}</cyan>:<cyan>{name}</cyan>:<cyan>{function}</cyan>:<cyan>{line}</cyan> | ' \
                        '<red>{extra[host]}</red> | ' \
                        '<level>{level.icon}{level: <5}</level> | ' \
                        '<level>{level.no}</level> | ' \
                        '<level>{extra[type]}</level> | ' \
                        '<level>{message}</level> '

    kafka_formatter = '{time:YYYY-MM-DD HH:mm:ss,SSS}| ' \
                      '[{extra[log_name]}] {module}:{name}:{function}:{line} | ' \
                      '{extra[host]} | ' \
                      '{process} | ' \
                      '{thread} | ' \
                      '{level: <5} | ' \
                      '{level.no} | ' \
                      '{extra[type]}| ' \
                      '{message} '

    def __init__(self):
        self.logger = logger

    def setter_log_handler(self, callback=None):
        assert callable(callback), 'callback must be a callable object'
        self.logger.add(callback, format=self.kafka_formatter)

    def get_logger(self, name=None):
        log_config = set_log_config(self.default_formatter)
        config = log_config.pop('default', {})
        if name:
            config['extra']['log_name'] = name
        self.logger.configure(**config)
        return self.logger

    @staticmethod
    def format(spider, meta):
        if hasattr(spider, 'logging_keys'):
            logging_txt = []
            for key in spider.logging_keys:
                if meta.get(key, None) is not None:
                    logging_txt.append(u'{0}:{1} '.format(key, meta[key]))
            logging_txt.append('successfully')
            return ' '.join(logging_txt)




import collections
import weakref

from collections.abc import Mapping


class CaselessDict(dict):

    __slots__ = ()

    def __init__(self, seq=None):
        super().__init__()
        if seq:
            self.update(seq)

    def __getitem__(self, key):
        return dict.__getitem__(self, self.normkey(key))

    def __setitem__(self, key, value):
        dict.__setitem__(self, self.normkey(key), self.normvalue(value))

    def __delitem__(self, key):
        dict.__delitem__(self, self.normkey(key))

    def __contains__(self, key):
        return dict.__contains__(self, self.normkey(key))
    has_key = __contains__

    def __copy__(self):
        return self.__class__(self)
    copy = __copy__

    def normkey(self, key):
        """Method to normalize dictionary key access"""
        return key.lower()

    def normvalue(self, value):
        """Method to normalize values prior to be setted"""
        return value

    def get(self, key, def_val=None):
        return dict.get(self, self.normkey(key), self.normvalue(def_val))

    def setdefault(self, key, def_val=None):
        return dict.setdefault(self, self.normkey(key), self.normvalue(def_val))

    def update(self, seq):
        seq = seq.items() if isinstance(seq, Mapping) else seq
        # print(seq)
        iseq = ((self.normkey(k), self.normvalue(v)) for k, v in seq)
        super().update(iseq)

    @classmethod
    def fromkeys(cls, keys, value=None):
        return cls((k, value) for k in keys)

    def pop(self, key, *args):
        return dict.pop(self, self.normkey(key), *args)


class LocalCache(collections.OrderedDict):
    """Dictionary with a finite number of keys.

    Older items expires first.
    """

    def __init__(self, limit=None):
        super().__init__()
        self.limit = limit

    def __setitem__(self, key, value):
        if self.limit:
            while len(self) >= self.limit:
                self.popitem(last=False)
        super().__setitem__(key, value)


class LocalWeakReferencedCache(weakref.WeakKeyDictionary):

    def __init__(self, limit=None):
        super().__init__()
        self.data = LocalCache(limit=limit)

    def __setitem__(self, key, value):
        try:
            super().__setitem__(key, value)
        except TypeError:
            pass  # key is not weak-referenceable, skip caching

    def __getitem__(self, key):
        try:
            return super().__getitem__(key)
        except (TypeError, KeyError):
            return None  # key is either not weak-referenceable or not cached


class SequenceExclude:
    """Object to test if an item is NOT within some sequence."""

    def __init__(self, seq):
        self.seq = seq

    def __contains__(self, item):
        return item not in self.seq



# encoding: utf-8
import json
import string
import time
import hashlib

from random import sample, randint, choice
from urllib.parse import urlsplit

from anti_useragent import UserAgent
from anti_header.exceptions import UserAgentError, NotFoundParamError, UnSupportMethodError
from anti_header.headers import Headers
from anti_header.utils import logging


class UsageHeader(object):
    _INSTANCE = None

    def __new__(cls, *args, **kwargs):
        """
        singer instance if set dry param is True then destory current instance and create a new instance
        :param dry: True|False
        :return cls
        """
        if not cls._INSTANCE:
            cls._INSTANCE = super().__new__(cls)
        if kwargs.get('dry', False) is True:
            return super().__new__(cls)
        return cls._INSTANCE

    def __init__(self, url: str = None,
                 rand_header: dict = None,
                 must_header: dict = None,
                 default_header: dict = None,
                 connection: bool = False,
                 browser: str = "random",
                 dry: bool = False,
                 logger=False, **kwargs):
        assert isinstance(logger, bool), "logger param must bool type"
        self.logger = logging.get_logger('anti_header') if logger else None
        self.url: str = url or "https://www.google.com/"
        self.kwargs = kwargs
        self.connection = connection
        self.browser = browser
        self.dry = dry

        self._headers: Headers = self.set_headers_default(default_header or {})
        self.headers_must: list = self.set_headers_must(must_header or {})
        self.headers_param: list = self.set_header_rand(rand_header or {})
        self._ua = self.__ua

    def __getitem__(self, rule: str):
        return self.__getattr__(rule)

    def __getattr__(self, rule: str):
        if rule == 'random':
            self.headers_must.append(self.user_agent())
            return getattr(self, "process")()
        elif rule == 'base':
            for param in self.headers_must:
                self._headers.update(param)
            self._headers.update(self.user_agent())
            return self._headers
        else:
            raise UnSupportMethodError

    @property
    def is_singleton(self):
        return not self.dry

    @property
    def __ua(self) -> UserAgent:
        return UserAgent(**self.kwargs)

    def set_headers_default(self, header: dict) -> Headers:
        _headers = Headers({})
        _header_default = [
            {'X-Forwarded-For': f'{randint(1, 254)}.{randint(1, 254)}.{randint(1, 254)}.{randint(1, 254)}'},
            {'connection': self.connection and 'keep-alive' or 'close'}
        ]
        _header_default.extend([{key: value} for key, value in header.items()])
        for default in _header_default:
            _headers.update(default)
        return _headers

    def set_headers_must(self, header: dict) -> list:
        _header_must = [
            {''.join(sample(string.ascii_lowercase, 5)): ''.join(sample(string.ascii_lowercase, 6))},
            {'referer': self.url},
        ]
        _header_must.extend([{key: value} for key, value in header.items()])
        return _header_must

    def set_header_rand(self, header: dict) -> list:
        parse_url = urlsplit(self.url)
        _header_params = [
            {'accept': '*/*'},
            {'accept-type': 'utf-8'},
            {'accept-encoding': 'gzip, deflate'},
            {'accept-language': 'zh-CN,zh;q=0.9,en;q=0.8'},
            {'authority': parse_url.netloc},
            {'cache-control': choice(['max-age=0', 'no-cache'])},
            {'cache-type': 'any'},
            {'content-from': 'google'},
            # {"content-type": "application/x-www-form-urlencoded"},

            {'cookie': self._md_hex},

            {'DNT': '1'},
            {'Host': parse_url.netloc},
            {'Origin': self.url},
            {'Origin-content': 'false'},
            {'referer-rec': 'ture'},
            {'pragma': 'no-cache'},
            {'sec-ch-ua': '" Not;A Brand";v="99", "Google Chrome";v="97", "Chromium";v="97"'},
            {'sec-ch-ua-mobile': '?0'},
            {'sec-ch-ua-platform': '"Windows"'},
            {'sec-fetch-site': 'same-origin'},
            {'sec-fetch-mode': 'navigate'},
            {'sec-fetch-user': '?1'},
            {'sec-fetch-dest': 'document'},
            {'TE': 'Trailers'},
            {'upgrade-type': 'none'},
            {'upgrade-insecure-requests': '1'},
        ]
        _header_params.extend([{key: value} for key, value in header.items()])
        return _header_params

    @property
    def _md_hex(self) -> str:
        _m = hashlib.md5()
        _m.update(''.join(sample(string.ascii_lowercase, 5)).encode())
        _m_hex = _m.hexdigest()
        return f'Hd_ert_{_m_hex}={time.time()}; Hd_ervt_{_m_hex}={time.time()}'

    def user_agent(self) -> dict:
        try:
            _ua = self._ua[self.browser]
        except UserAgentError:
            _ua = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36' \
                  ' (KHTML, like Gecko) Chrome/98.0.4758.102 Safari/537.36'
        return {"User-Agent": _ua}

    def process(self) -> Headers:
        new_header_params = sample(self.headers_param, randint(0, len(self.headers_param)))
        for param in new_header_params + self.headers_must:
            self._headers.update(param)
        if self.logger is not None:
            self.logger.debug('[Headers]: {}', str(self._headers.to_unicode_dict()))
        return self._headers


Header = UsageHeader
__version__ = '0.0.4'

VERSION = __version__


__all__ = [
    Header,
    UsageHeader,
    VERSION
]



from __future__ import unicode_literals

import os
import io

from setuptools import setup


def list_dir(dir):
    result = [dir]
    for file in os.listdir(dir):
        if os.path.isdir(os.path.join(dir, file)):
            result.extend(list_dir(os.path.join(dir, file)))
    return result


def read(*parts):
    filename = os.path.join(os.path.abspath(os.path.dirname(__file__)), *parts)

    with io.open(filename, encoding='utf-8', mode='rt') as fp:
        return fp.read()


NAME = "anti_header"
PACKAGES = list_dir('anti_header')
DESCRIPTION = "fake chrome, firefox, opera browser header anti header"
# LONG_DESCRIPTION = read('README.md')
LONG_DESCRIPTION = ''
URL = "https://github.com/ihandmine/anti-header.git"
AUTHOR = "handmine"
AUTHOR_EMAIL = "handmine@outlook.com"
VERSION = "0.0.4"
LICENSE = "MIT"

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
    ],
    url=URL,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    license=LICENSE,
    packages=PACKAGES,
    include_package_data=True,
    zip_safe=True,
    install_requires=[
        "anti-useragent",
    ],
    keywords=[
        'header', 'headers', 'crawl header',
        'fake', 'fake header', 'browser', 'spider header',
        'anti', 'anti header', 'scrapy header'
    ],
)



from anti_header.utils.datatypes import CaselessDict
from anti_header.utils.log import LogFormatter
from anti_header.utils.python import *
from anti_header.utils.http import headers_dict_to_raw

logging = LogFormatter()
